/* *****************************************************************************
 *
 * Filename: MainFrame.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  29 November 2002, 16:45
 *
 * Purpose:  The Main Window.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker;

import java.io.*;
import java.text.*;
import java.util.*;
import java.beans.*;
import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.print.*;
import javax.swing.*;
import javax.print.PrintService;
import java.beans.Customizer;

import com.tsac.calmaker.beans.*;

/**
 *
 * @author  Robert
 */
public class MainFrame
    extends javax.swing.JFrame
{
    // Vesion number (must NOT be final)
    public static String version = "1.1.0";
    
    // Constants
    public static final int BLANK_CALENDAR = 0;
    public static final int TABLE_CALENDAR = 1;
    public static final int LIST_CALENDAR  = 2;
    public static final int PANORAMA_CALENDAR = 3;
    public static final int COVERSHEET     = 4;
    
    public static double scale = 1.0;     // Zoom factor for normal display
    
    // Drag and Drop stuff
    private Vector dropSources = new Vector();
    
    // Cached values
    private ResourceBundle calendarMaker = ResourceBundle.getBundle( "com/tsac/calmaker/CalendarMaker" );
    private Icon blankIcon = new ImageIcon( getClass().getResource( "/com/tsac/calmaker/blank.gif" ) );
    
    // State
    private PageFormat _pageFormat = null;  // Save page setup info
    private PrintService _printer = null;   // Save the selected printer
    private Dimension _pageSize;            // Size for scrollpane and panel
    private File _filename = null;          // Filename of current calendar
    private int _startYear;                 // Year that yearCmb starts at
    private PropertiesDialog _props = null; // Properties dialog
    private Customizer _gblayoutCustom = new GridBagLayoutCustomizer();
    private Customizer _nulllayoutCustom = new NullLayoutCustomizer();
    
    // Temp fix
    private CalendarPanel _calendar;        // The current CalendarPanel
    

    /** Creates new form MainFrame */
    public MainFrame()
    {
        // Get the default page margins
        actionPageSetup( false );
        
        // Build the Frame
        initComponents();
        
        // Add all the beans to the edit toolbar
        createEditToolbarButtons();
                        
        // Populate Month ComboBox
        String[] months = new SimpleDateFormat().getDateFormatSymbols().getMonths();
        for( int n = 0; n < months.length-1; n++ )
            monthCmb.addItem( months[n] );
        
        // Populate Year ComboBox
        Calendar now = Calendar.getInstance();
        _startYear = now.get( Calendar.YEAR );
        for( int year = _startYear; year < _startYear + 5; year++ )
            yearCmb.addItem( Integer.toString( year ) );
        
        // Populate Zoom ComboBox
        zoomCmb.addItem( "100%" );
        zoomCmb.addItem( "75%" );
        zoomCmb.addItem( "50%" );
        zoomCmb.addItem( "25%" );
        
        // Set default menu and toolbar item states
        updateActionsWithSelection( panel.getSelection() != null );
        
        // Set default scale on the panel
        panel.setZoom( scale );
        
        // Add default page - will calculate page size
        actionNew( TABLE_CALENDAR );

        // Set size of Frame
        Dimension sz = getContentPane().getPreferredSize();
        setSize( sz.width + 8, sz.height + menubar.getHeight() + 34 );
        
        // Set the Frame's icon
        ImageIcon icon = new ImageIcon( getClass().getResource( "/com/tsac/calmaker/icons/calmaker.gif" ) );
        this.setIconImage( icon.getImage() );
        
        // Setup accessibility stuff that isn't set automatically or with code-gen
        this.getAccessibleContext().setAccessibleDescription( calendarMaker.getString( "Taurus_CalendarMaker_Desc" ) );
        monthCmb.getAccessibleContext().setAccessibleDescription( calendarMaker.getString( "Month_Tooltip" ) );
        yearCmb.getAccessibleContext().setAccessibleDescription( calendarMaker.getString( "Year_Tooltip" ) );
        zoomCmb.getAccessibleContext().setAccessibleDescription( calendarMaker.getString( "Zoom_Tooltip" ) );
        
        // Fix the accessible names of toolbar buttons to NOT contain "dot dot dot"
        Utilities.fixAccessibleButtonNames( mainToolbar );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        editToolbarBtnGrp = new javax.swing.ButtonGroup();
        scrollpane = new javax.swing.JScrollPane();
        panel = new com.tsac.calmaker.beans.DateablePanel();
        toolbarPnl = new javax.swing.JPanel();
        mainToolbar = new javax.swing.JToolBar();
        newBtn = new JButton( newBlankCalendarAction );
        openBtn = new JButton( openAction );
        saveBtn = new JButton( saveAction );
        printBtn = new JButton( printAction );
        deleteBtn = new JButton( deleteAction );
        propsBtn = new JButton( propertiesAction );
        prevMonthBtn = new JButton( prevMonthAction );
        nextMonthBtn = new JButton( nextMonthAction );
        monthCmb = new javax.swing.JComboBox();
        yearCmb = new javax.swing.JComboBox();
        zoomCmb = new javax.swing.JComboBox();
        editToolbar = new javax.swing.JToolBar();
        noneBtn = new javax.swing.JToggleButton();
        menubar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        newMenu = new javax.swing.JMenu();
        newCoversheetItem = new javax.swing.JMenuItem();
        newTableCalItem = new javax.swing.JMenuItem();
        newListCalItem = new javax.swing.JMenuItem();
        newPanoCalItem = new javax.swing.JMenuItem();
        newBlankCalItem = new JMenuItem( newBlankCalendarAction );
        openItem = new JMenuItem( openAction );
        saveItem = new JMenuItem( saveAction );
        saveAsItem = new JMenuItem( saveAsAction );
        pageSetupItem = new JMenuItem( pageSetupAction );
        printItem = new JMenuItem( printAction );
        exitItem = new JMenuItem( exitAction );
        editMenu = new javax.swing.JMenu();
        deleteItem = new JMenuItem( deleteAction );
        clearSelItem = new JMenuItem( clearSelectionAction );
        nextMonthItem = new JMenuItem( nextMonthAction );
        prevMonthItem = new JMenuItem( prevMonthAction );
        propsItem = new JMenuItem( propertiesAction );
        helpMenu = new javax.swing.JMenu();
        aboutItem = new javax.swing.JMenuItem();

        setTitle(calendarMaker.getString("Taurus_CalendarMaker"));
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                exitForm(evt);
            }
        });

        panel.setLayout(null);

        panel.addDateableSelectionListener(new com.tsac.calmaker.beans.DateableSelectionListener()
        {
            public void selectionChanged(com.tsac.calmaker.beans.DateableSelectionEvent evt)
            {
                panelSelectionChanged(evt);
            }
            public void importTemplate(com.tsac.calmaker.beans.DateableSelectionEvent evt)
            {
                panelImportTemplate(evt);
            }
            public void doubleClicked(com.tsac.calmaker.beans.DateableSelectionEvent evt)
            {
                panelDoubleClicked(evt);
            }
        });

        scrollpane.setViewportView(panel);

        getContentPane().add(scrollpane, java.awt.BorderLayout.CENTER);

        toolbarPnl.setLayout(new java.awt.BorderLayout());

        mainToolbar.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED, null, (java.awt.Color) javax.swing.UIManager.getDefaults().get("Button.background"), null, (java.awt.Color) javax.swing.UIManager.getDefaults().get("Button.background")));
        mainToolbar.setFloatable(false);
        mainToolbar.setRollover(true);
        mainToolbar.setToolTipText(calendarMaker.getString("Main_Toolbar_Tooltip"));
        mainToolbar.setName(calendarMaker.getString("Main"));
        newBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/new.gif")));
        newBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        newBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "New" ) );
        mainToolbar.add(newBtn);

        openBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/open.gif")));
        openBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        openBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Open_" ) );
        mainToolbar.add(openBtn);

        saveBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/save.gif")));
        saveBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        saveBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Save_" ) );
        mainToolbar.add(saveBtn);

        mainToolbar.addSeparator();
        printBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/print.gif")));
        printBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        printBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Print_" ) );
        mainToolbar.add(printBtn);

        mainToolbar.addSeparator();
        deleteBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/delete.gif")));
        deleteBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        deleteBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Delete" ) );
        mainToolbar.add(deleteBtn);

        propsBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/props.gif")));
        propsBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        propsBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Properties_" ) );
        mainToolbar.add(propsBtn);

        mainToolbar.addSeparator();
        prevMonthBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/prev.gif")));
        prevMonthBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        prevMonthBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Prev_Month" ) );
        mainToolbar.add(prevMonthBtn);

        nextMonthBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/next.gif")));
        nextMonthBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        nextMonthBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Next_Month" ) );
        mainToolbar.add(nextMonthBtn);

        monthCmb.setToolTipText(calendarMaker.getString("Month_Tooltip"));
        monthCmb.getAccessibleContext().setAccessibleName( calendarMaker.getString("Month_Name") );
        monthCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                monthCmbActionPerformed(evt);
            }
        });

        mainToolbar.add(monthCmb);

        yearCmb.setToolTipText(calendarMaker.getString("Year_Tooltip"));
        yearCmb.getAccessibleContext().setAccessibleName( calendarMaker.getString("Year_Name") );
        yearCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                yearCmbActionPerformed(evt);
            }
        });

        mainToolbar.add(yearCmb);

        zoomCmb.setToolTipText(calendarMaker.getString("Zoom_Tooltip"));
        zoomCmb.setMaximumSize(new java.awt.Dimension(75, 32768));
        zoomCmb.setPreferredSize(new java.awt.Dimension(75, 25));
        zoomCmb.getAccessibleContext().setAccessibleName( calendarMaker.getString("Zoom_Name") );
        zoomCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                zoomCmbActionPerformed(evt);
            }
        });

        mainToolbar.add(zoomCmb);

        toolbarPnl.add(mainToolbar, java.awt.BorderLayout.NORTH);

        editToolbar.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED, null, (java.awt.Color) javax.swing.UIManager.getDefaults().get("Button.background"), null, (java.awt.Color) javax.swing.UIManager.getDefaults().get("Button.background")));
        editToolbar.setFloatable(false);
        editToolbar.setRollover(true);
        editToolbar.setToolTipText(calendarMaker.getString("Edit_Toolbar_Tooltip"));
        editToolbar.setName(calendarMaker.getString("Edit"));
        noneBtn.setSelected(true);
        noneBtn.setText("none");
        editToolbarBtnGrp.add(noneBtn);
        noneBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        noneBtn.setVisible( false );
        editToolbar.add(noneBtn);

        toolbarPnl.add(editToolbar, java.awt.BorderLayout.SOUTH);

        getContentPane().add(toolbarPnl, java.awt.BorderLayout.NORTH);

        fileMenu.setMnemonic(calendarMaker.getString("File_Mnemonic").charAt(0));
        fileMenu.setText(calendarMaker.getString("File"));
        newMenu.setMnemonic(calendarMaker.getString("New_Mnemonic").charAt(0));
        newMenu.setText(calendarMaker.getString("New"));
        newMenu.setIcon(blankIcon);
        newCoversheetItem.setMnemonic(calendarMaker.getString( "Coversheet_Mnemonic" ).charAt(0));
        newCoversheetItem.setText(calendarMaker.getString("Coversheet"));
        newCoversheetItem.setIcon(blankIcon);
        newCoversheetItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                newCoversheetItemActionPerformed(evt);
            }
        });

        newMenu.add(newCoversheetItem);
        newTableCalItem.setMnemonic(calendarMaker.getString("Table_Calendar_Mnemonic").charAt(0));
        newTableCalItem.setText(calendarMaker.getString("Table_Calendar"));
        newTableCalItem.setIcon(blankIcon);
        newTableCalItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                newTableCalItemActionPerformed(evt);
            }
        });

        newMenu.add(newTableCalItem);
        newListCalItem.setMnemonic(calendarMaker.getString("List_Calendar_Mnemonic").charAt(0));
        newListCalItem.setText(calendarMaker.getString("List_Calendar"));
        newListCalItem.setIcon(blankIcon);
        newListCalItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                newListCalItemActionPerformed(evt);
            }
        });

        newMenu.add(newListCalItem);
        newPanoCalItem.setMnemonic(calendarMaker.getString("Panorama_Calendar_Mnemonic").charAt(0));
        newPanoCalItem.setText(calendarMaker.getString("Panorama_Calendar"));
        newPanoCalItem.setIcon(blankIcon);
        newPanoCalItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                newPanoCalItemActionPerformed(evt);
            }
        });

        newMenu.add(newPanoCalItem);
        newBlankCalItem.setMnemonic(calendarMaker.getString("Blank_Calendar_Mnemonic").charAt(0));
        newBlankCalItem.setText(calendarMaker.getString("Blank_Calendar"));
        newBlankCalItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                newBlankCalItemActionPerformed(evt);
            }
        });

        newMenu.add(newBlankCalItem);
        fileMenu.add(newMenu);
        openItem.setMnemonic(calendarMaker.getString("Open_Mnemonic").charAt(0));
        openItem.setText(calendarMaker.getString("Open_"));
        fileMenu.add(openItem);
        fileMenu.addSeparator();
        saveItem.setMnemonic(calendarMaker.getString("Save_Mnemonic").charAt(0));
        saveItem.setText(calendarMaker.getString("Save_"));
        fileMenu.add(saveItem);
        saveAsItem.setMnemonic(calendarMaker.getString("Save_As_Mnemonic").charAt(0));
        saveAsItem.setText(calendarMaker.getString("Save_As_"));
        saveAsItem.setIcon(blankIcon);
        fileMenu.add(saveAsItem);
        fileMenu.addSeparator();
        pageSetupItem.setMnemonic(calendarMaker.getString("Page_Setup_Mnemonic").charAt(0));
        pageSetupItem.setText(calendarMaker.getString("Page_Setup_"));
        pageSetupItem.setIcon(blankIcon);
        fileMenu.add(pageSetupItem);
        printItem.setMnemonic(calendarMaker.getString("Print_Mnemonic").charAt(0));
        printItem.setText(calendarMaker.getString("Print_"));
        fileMenu.add(printItem);
        fileMenu.addSeparator();
        exitItem.setMnemonic(calendarMaker.getString("Exit_Mnemonic").charAt(0));
        exitItem.setText(calendarMaker.getString("Exit"));
        exitItem.setIcon(blankIcon);
        fileMenu.add(exitItem);
        menubar.add(fileMenu);
        editMenu.setMnemonic(calendarMaker.getString("Edit_Mnemonic").charAt(0));
        editMenu.setText(calendarMaker.getString("Edit"));
        deleteItem.setMnemonic(calendarMaker.getString("Delete_Mnemonic").charAt(0));
        deleteItem.setText(calendarMaker.getString("Delete"));
        editMenu.add(deleteItem);
        clearSelItem.setMnemonic(calendarMaker.getString("Clear_Selection_Mnemonic").charAt(0));
        clearSelItem.setText(calendarMaker.getString("Clear_Selection"));
        clearSelItem.setIcon(blankIcon);
        editMenu.add(clearSelItem);
        editMenu.addSeparator();
        nextMonthItem.setMnemonic(calendarMaker.getString( "Next_Month_Mnemonic" ).charAt(0));
        nextMonthItem.setText(calendarMaker.getString("Next_Month"));
        editMenu.add(nextMonthItem);
        prevMonthItem.setMnemonic(calendarMaker.getString( "Prev_Month_Mnemonic" ).charAt(0));
        prevMonthItem.setText(calendarMaker.getString("Prev_Month"));
        editMenu.add(prevMonthItem);
        editMenu.addSeparator();
        propsItem.setMnemonic(calendarMaker.getString("Properties_Mnemonic").charAt(0));
        propsItem.setText(calendarMaker.getString("Properties_"));
        editMenu.add(propsItem);
        menubar.add(editMenu);
        //menubar.add( Box.createHorizontalGlue() );
        helpMenu.setMnemonic(calendarMaker.getString("Help_Mnemonic").charAt(0));
        helpMenu.setText(calendarMaker.getString("Help"));
        aboutItem.setMnemonic(calendarMaker.getString("About_Mnemonic").charAt(0));
        aboutItem.setText(calendarMaker.getString("About_"));
        aboutItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                aboutItemActionPerformed(evt);
            }
        });

        helpMenu.add(aboutItem);
        menubar.add(helpMenu);
        setJMenuBar(menubar);

        pack();
    }//GEN-END:initComponents

    private void newCoversheetItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_newCoversheetItemActionPerformed
    {//GEN-HEADEREND:event_newCoversheetItemActionPerformed
        actionNew( COVERSHEET );
    }//GEN-LAST:event_newCoversheetItemActionPerformed

    private void newPanoCalItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_newPanoCalItemActionPerformed
    {//GEN-HEADEREND:event_newPanoCalItemActionPerformed
        actionNew( PANORAMA_CALENDAR );
    }//GEN-LAST:event_newPanoCalItemActionPerformed

    private void panelDoubleClicked(com.tsac.calmaker.beans.DateableSelectionEvent evt)//GEN-FIRST:event_panelDoubleClicked
    {//GEN-HEADEREND:event_panelDoubleClicked
        if( evt.getComponent() != null )
            actionProperties();
    }//GEN-LAST:event_panelDoubleClicked

    private void panelImportTemplate(com.tsac.calmaker.beans.DateableSelectionEvent evt)//GEN-FIRST:event_panelImportTemplate
    {//GEN-HEADEREND:event_panelImportTemplate
        performLoad( evt.getFile().toString() );
    }//GEN-LAST:event_panelImportTemplate

    private void panelSelectionChanged(com.tsac.calmaker.beans.DateableSelectionEvent evt)//GEN-FIRST:event_panelSelectionChanged
    {//GEN-HEADEREND:event_panelSelectionChanged
        updateActionsWithSelection( evt.getComponent() != null );
        updatePropertiesDialog();
    }//GEN-LAST:event_panelSelectionChanged

    private void yearCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_yearCmbActionPerformed
    {//GEN-HEADEREND:event_yearCmbActionPerformed
        setDate( _startYear + yearCmb.getSelectedIndex(), monthCmb.getSelectedIndex() );
    }//GEN-LAST:event_yearCmbActionPerformed

    private void newBlankCalItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_newBlankCalItemActionPerformed
    {//GEN-HEADEREND:event_newBlankCalItemActionPerformed
        actionNew( BLANK_CALENDAR );
    }//GEN-LAST:event_newBlankCalItemActionPerformed

    private void aboutItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_aboutItemActionPerformed
    {//GEN-HEADEREND:event_aboutItemActionPerformed
        new AboutDialog( this, true ).setVisible( true );
    }//GEN-LAST:event_aboutItemActionPerformed

    private void newListCalItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_newListCalItemActionPerformed
    {//GEN-HEADEREND:event_newListCalItemActionPerformed
        actionNew( LIST_CALENDAR );
    }//GEN-LAST:event_newListCalItemActionPerformed

    private void newTableCalItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_newTableCalItemActionPerformed
    {//GEN-HEADEREND:event_newTableCalItemActionPerformed
        actionNew( TABLE_CALENDAR );
    }//GEN-LAST:event_newTableCalItemActionPerformed

    private void zoomCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_zoomCmbActionPerformed
    {//GEN-HEADEREND:event_zoomCmbActionPerformed
        switch( zoomCmb.getSelectedIndex() )
        {
        case 0: scale = 1.0; break;
        case 1: scale = 0.75; break;
        case 2: scale = 0.5; break;
        case 3: scale = 0.25; break;
        default: scale = 1.0; break;
        }
        MainFrame.scale = scale;    // cludge
        panel.setZoom( scale );
        repaint();
    }//GEN-LAST:event_zoomCmbActionPerformed

    private void monthCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_monthCmbActionPerformed
    {//GEN-HEADEREND:event_monthCmbActionPerformed
        setDate( _startYear + yearCmb.getSelectedIndex(), monthCmb.getSelectedIndex() );
    }//GEN-LAST:event_monthCmbActionPerformed
                
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt)//GEN-FIRST:event_exitForm
    {//GEN-HEADEREND:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main( String args[] )
    {
        File template = null;
        
        if( args.length > 0 )
        {
            if( args.length == 1 )
            {
                template = new File( args[0] );
            }
            else
            {
                System.out.println( "Taurus Calendar Maker V" + version );
                System.out.println( "Usage: <command> [filename]" );
                System.exit( 1 );
            }
        }
        
        SplashWindow splash = new SplashWindow();
        splash.setVisible( true );
        
        long before = System.currentTimeMillis();
        
        // Change L&F to system L&F if none specified - splash needs no L&F
        if( System.getProperty( "swing.defaultlaf" ) == null )
        {
            try {
                UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
            } catch( Exception e ) {
                System.out.println( e );
            }
        }
        
        MainFrame window = new MainFrame();
        
        long after = System.currentTimeMillis();
        
        // Make sure splash remains visible for at least 2 seconds
        if( after - before < 2000 ) {
            try { Thread.sleep( 2000 - (after - before) ); } catch( InterruptedException e ) {}
        }
        
        window.setVisible( true );
        
        splash.setVisible( false );
        splash = null;
        
        if( template != null )
        {
            if( window.performLoad( template.toString() ) )
                window.setFilename( template );
        }
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem openItem;
    private javax.swing.JComboBox yearCmb;
    private javax.swing.JMenuItem newCoversheetItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JScrollPane scrollpane;
    private javax.swing.JPanel toolbarPnl;
    private javax.swing.JButton openBtn;
    private javax.swing.JButton deleteBtn;
    private javax.swing.JMenuItem saveItem;
    private javax.swing.JButton printBtn;
    private javax.swing.JMenuItem newPanoCalItem;
    private javax.swing.JComboBox zoomCmb;
    private javax.swing.JMenuItem propsItem;
    private javax.swing.JMenuItem pageSetupItem;
    private javax.swing.JMenuItem printItem;
    private javax.swing.JButton saveBtn;
    private javax.swing.JMenu newMenu;
    private javax.swing.JMenuItem deleteItem;
    private javax.swing.JButton newBtn;
    private javax.swing.ButtonGroup editToolbarBtnGrp;
    private javax.swing.JComboBox monthCmb;
    private javax.swing.JMenuBar menubar;
    private javax.swing.JMenuItem newTableCalItem;
    private javax.swing.JToolBar editToolbar;
    private javax.swing.JMenuItem exitItem;
    private javax.swing.JButton nextMonthBtn;
    private javax.swing.JButton prevMonthBtn;
    private javax.swing.JMenuItem newBlankCalItem;
    private javax.swing.JButton propsBtn;
    private javax.swing.JMenu editMenu;
    private javax.swing.JToggleButton noneBtn;
    private com.tsac.calmaker.beans.DateablePanel panel;
    private javax.swing.JMenuItem newListCalItem;
    private javax.swing.JToolBar mainToolbar;
    private javax.swing.JMenuItem saveAsItem;
    private javax.swing.JMenuItem aboutItem;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JMenuItem prevMonthItem;
    private javax.swing.JMenuItem nextMonthItem;
    private javax.swing.JMenuItem clearSelItem;
    // End of variables declaration//GEN-END:variables
    
    
    private class CalendarFileFilter
        extends javax.swing.filechooser.FileFilter
    {
        public boolean accept( java.io.File f )
        {
            return f.getName().endsWith( ".cal" ) || f.getName().endsWith( ".xml" ) || f.isDirectory();
        }
        
        public String getDescription()
        {
            return calendarMaker.getString( "Calendar_files_filter" );
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Actions for File menu and left portion of Main toolbar

    private class NewBlankCalendarAction
        extends AbstractAction
    {
        public NewBlankCalendarAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/new.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Blank_Calendar_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_N, InputEvent.CTRL_MASK ) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionNew( BLANK_CALENDAR );
        }
    }
    
    private NewBlankCalendarAction newBlankCalendarAction = new NewBlankCalendarAction();

    
    private class OpenAction
        extends AbstractAction
    {
        public OpenAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/open.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Open_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_O, InputEvent.CTRL_MASK ) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionLoad();
        }
    }
    
    private OpenAction openAction = new OpenAction();
    
    
    private class SaveAction
        extends AbstractAction
    {
        public SaveAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/save.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Save_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionSave();
        }
    }
    
    private SaveAction saveAction = new SaveAction();
    
    
    private class SaveAsAction
        extends AbstractAction
    {
        public SaveAsAction()
        {
            super();
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Save_As_Tooltip" ) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionSaveAs();
        }
    }
    
    private SaveAsAction saveAsAction = new SaveAsAction();

    
    private class PageSetupAction
        extends AbstractAction
    {
        public PageSetupAction()
        {
            super();
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Page_Setup_Tooltip" ) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionPageSetup( true );
        }
    }
    
    private PageSetupAction pageSetupAction = new PageSetupAction();

    
    private class PrintAction
        extends AbstractAction
    {
        public PrintAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/print.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Print_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionPrint();
        }
    }
    
    private PrintAction printAction = new PrintAction();

    
    private class ExitAction
        extends AbstractAction
    {
        public ExitAction()
        {
            super();
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Exit_Tooltip" ) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            System.exit(0);
        }
    }
    
    private ExitAction exitAction = new ExitAction();
    
    ////////////////////////////////////////////////////////////////////////////
    // Actions for Edit menu and centre portion of Main toolbar

    private class ClearSelectionAction
        extends AbstractAction
    {
        public ClearSelectionAction()
        {
            super();
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Clear_Selection_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            panel.setSelection( null );
        }
    }

    private ClearSelectionAction clearSelectionAction = new ClearSelectionAction();
    
    
    private class PropertiesAction
        extends AbstractAction
    {
        public PropertiesAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/props.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Properties_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionProperties();
        }
    }
    
    private PropertiesAction propertiesAction = new PropertiesAction();

    
    private class DeleteAction
        extends AbstractAction
    {
        public DeleteAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/delete.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Delete_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            actionDelete();
        }
    }
    
    private DeleteAction deleteAction = new DeleteAction();
    
    
    private class PrevMonthAction
        extends AbstractAction
    {
        public PrevMonthAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/prev.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Prev_Month_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, InputEvent.CTRL_MASK) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            // Set selected index - this takes care of enabling/disabling buttons
            monthCmb.setSelectedIndex(
                monthCmb.getSelectedIndex() > 0
                ? (monthCmb.getSelectedIndex() - 1)
                : 0 );
        }
    }
    
    private PrevMonthAction prevMonthAction = new PrevMonthAction();
    
    
    private class NextMonthAction
        extends AbstractAction
    {
        public NextMonthAction()
        {
            super( null, new ImageIcon( MainFrame.this.getClass().getResource( "/com/tsac/calmaker/next.gif" ) ) );
            putValue( SHORT_DESCRIPTION, calendarMaker.getString( "Next_Month_Tooltip" ) );
            putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, InputEvent.CTRL_MASK) );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            // Set selected index - this takes care of enabling/disabling buttons
            monthCmb.setSelectedIndex(
                monthCmb.getSelectedIndex() < (monthCmb.getItemCount() - 1)
                ? (monthCmb.getSelectedIndex() + 1)
                : (monthCmb.getItemCount() - 1) );
        }
    }
    
    private NextMonthAction nextMonthAction = new NextMonthAction();
    
    ////////////////////////////////////////////////////////////////////////////
    // Actions for Insert menu and Edit toolbar
        
    private class SelectBeanAction
        extends AbstractAction
    {
        private Class _beanClass;
        
        public SelectBeanAction( Class beanClass )
        {
            _beanClass = beanClass;
            
            try
            {
                BeanInfo bi = Introspector.getBeanInfo( beanClass );
                putValue( SMALL_ICON, new ImageIcon( bi.getIcon( BeanInfo.ICON_COLOR_16x16 ) ) );
            }
            catch( IntrospectionException e )
            {
                e.printStackTrace();
                putValue( SMALL_ICON, MainFrame.this.getClass().getResource( "/com/tsac/calmaker/icons/unknownC16.gif" ) );                
            }
            
            String fmt = calendarMaker.getString( "Bean_Tooltip_Format" );
            String tooltip = MessageFormat.format( fmt, new Object[] { beanNameFromClass(beanClass) } );
            putValue( SHORT_DESCRIPTION, tooltip );
        }
        
        public void actionPerformed( java.awt.event.ActionEvent e )
        {
            panel.setSelectedBean( _beanClass );
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    
    // Change date of CalendarBean and keep toolbar in sync
    private void setDate( int year, int month )
    {
        panel.setDate( year, month );
        
        prevMonthAction.setEnabled( month > 0 );
        nextMonthAction.setEnabled( month < monthCmb.getItemCount() - 1 );
    }
    
    
    // Create the buttons on the Edit toolbar
    private void createEditToolbarButtons()
    {
        Class[] beans = new Class[] {
            TableCalendar.class, ListCalendar.class, PanoramaCalendar.class,
            MonthImage.class, StaticImage.class,
            MonthTitle.class, YearTitle.class, StaticTitle.class
        };
        
        for( int n = 0; n < beans.length; n++ )
        {
            // Create a button and add it to the toolbar
            JToggleButton button = new JToggleButton( new SelectBeanAction( beans[n] ) );
            editToolbarBtnGrp.add( button );
            button.setMargin( new java.awt.Insets(1, 1, 1, 1) );
            button.getAccessibleContext().setAccessibleName( beanNameFromClass( beans[n] ) );
            editToolbar.add( button );

            // Create drop source for the button
            dropSources.addElement( new MFDropSource( button, ((ImageIcon) button.getIcon()).getImage(), beans[n] ) );
        }
    }
    
    
    // Change the filename of the currently loaded file and update frame title
    private void setFilename( File filename )
    {
        _filename = filename;
        setTitle( calendarMaker.getString( "Taurus_CalendarMaker" ) + " - " + filename.toString() );
    }
    
    
    private void actionLoad()
    {
        JFileChooser opendlg = new JFileChooser( "." );
        javax.swing.filechooser.FileFilter calFilter;
        opendlg.addChoosableFileFilter( calFilter = new CalendarFileFilter() );
        opendlg.setFileFilter( calFilter );

        if( opendlg.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION )
        {
            File file = opendlg.getSelectedFile();
            
            if( performLoad( file.toString() ) )
                setFilename( file );
        }
    }
    
    
    private boolean performLoad( String filename )
    {
        Object o = null;
        
        // Load the list of CalendarPanel objects
        try
        {
            BufferedInputStream in = new BufferedInputStream( new FileInputStream( filename ) );
            XMLDecoder decoder = new XMLDecoder( in );
            o = decoder.readObject();
            decoder.close();
        }
        catch( IOException e )
        {
            e.printStackTrace();
            return false;
        }
        
        // If loaded successfully add CalendarPanels to panel
        if( (o != null) && (o instanceof ArrayList) )
        {
            panel.removeAll();
            
            ArrayList l = (ArrayList) o;
            
            for( Iterator i = l.iterator(); i.hasNext(); )
            {
                Object obj = i.next();
                
                if( obj instanceof CalendarPanel )
                {
                    CalendarPanel cp = (CalendarPanel) obj;
                    panel.add( cp );
                    panel.setNumPagesWidth( cp.getNumPagesWidth() );
                    panel.setActualSize( cp.getPreferredSize() );
                }
            }
        
            panel.repaint();
            
            setDate( _startYear + yearCmb.getSelectedIndex(), monthCmb.getSelectedIndex() );		            
        }
        
        return true;
    }
    
    
    private void actionSave()
    {
        if( _filename == null )
        {
            actionSaveAs();
            return;
        }
        
        performSave( _filename.toString() );
    }
    
    
    private void actionSaveAs()
    {
        JFileChooser savedlg = new JFileChooser( "." );
        javax.swing.filechooser.FileFilter calFilter;
        savedlg.addChoosableFileFilter( calFilter = new CalendarFileFilter() );
        savedlg.setFileFilter( calFilter );

        if( savedlg.showSaveDialog( this ) == JFileChooser.APPROVE_OPTION )
        {
            setFilename( savedlg.getSelectedFile() );
            performSave( _filename.toString() );
        }
    }
    
    
    private void performSave( String filename )
    {
        // Temporary list to hold beans to save
        ArrayList l = new ArrayList();

        for( int n = 0; n < panel.getComponentCount(); n++ )
            l.add( panel.getComponent(n) );
        
        // Now do the save
        try
        {
            BufferedOutputStream out = new BufferedOutputStream( new FileOutputStream( filename ) );
            XMLEncoder encoder = new XMLEncoder( out );
            encoder.writeObject( l );
            encoder.close();
        }
        catch( IOException e )
        {
            e.printStackTrace();
        }
    }
    
    
    private void actionPageSetup( boolean show )
    {
        PrinterJob job = PrinterJob.getPrinterJob();
        
        // Set the selected printer (if there is one)
        if( _printer != null ) {
            try {
                job.setPrintService( _printer );
            } catch( PrinterException e ) {
            }
        }
        
        // Is this the first time we have been called?
        if( _pageFormat == null )
        {
            PageFormat pf = job.defaultPage();
            
            // Modify a copy of the default paper to have minimal margins
            Paper p = pf.getPaper();
            p.setImageableArea( 0, 0, p.getWidth(), p.getHeight() );
            pf.setPaper( p );
            
            // Use this new PageFormat as the default for the Page Setup Dialog
            _pageFormat = job.validatePage( pf );
        }
        
        job.setPrintable( panel );

        // Are we supposed to show the page setup dialog?
        if( show )
        {
            PageFormat oldPageFormat = _pageFormat;

            // Save modified (or unchanged) page setup info
            _pageFormat = job.pageDialog( oldPageFormat );

            // Was dialog okayed?
            if( _pageFormat != oldPageFormat )
            {
                // Save selected printer
               _printer = job.getPrintService();

               // Check the margins are okay for current printer
               _pageFormat = job.validatePage( _pageFormat );
               
                // Update page size
                calcPageSize();
            }
        }
    }
    
    
    private void actionPrint()
    {
        PrinterJob job = PrinterJob.getPrinterJob();

        // Set the selected printer (if there is one)
        if( _printer != null ) {
            try {
                job.setPrintService( _printer );
            } catch( PrinterException e ) {
            }
        }
        
        if( _pageFormat != null )
            job.setPrintable( panel, _pageFormat );
        else
            job.setPrintable( panel );
        
        // Check if cancel was pressed
        if( !job.printDialog() )
            return;
        
         // Save selected printer
        _printer = job.getPrintService();
        
        // Now do the printing...
        try {
            job.print();
        } catch( Exception e ) {
            e.printStackTrace();
        }
    }
    
    
    private void actionDelete()
    {
        Component selection = panel.getSelection();
        if( selection == null )
            return;
        
        Container parent = selection.getParent();
        if( parent == null )
            return;
        
        // About to delete selection, so get rid of panels reference to it
        panel.setSelection( null );
        
        // Remove selection from its parent and redraw
        parent.remove( selection );
        parent.validate();
        parent.repaint();
    }
    
    
    private void actionProperties()
    {
        // Create the properties dialog if we haven't done so
        if( _props == null )
            _props = new PropertiesDialog();

        // Make sure dialog is visible if selection is valid
        if( updatePropertiesDialog() )
            _props.setVisible( true );
    }
    
    
    private boolean updatePropertiesDialog()
    {
        if( _props == null )
            return false;

        Component c = panel.getSelection();
        Customizer propsCustom = propsCustomizerFromComponent( c );
        Customizer layoutCustom = layoutCustomizerFromComponent( c );

        // Set the title of the property editor to reflect bean name
        if( c != null )
            _props.setTitle( calendarMaker.getString("Properties") + " - " + beanNameFromClass( c.getClass() ) );
        else
            _props.setTitle( calendarMaker.getString("Properties") );
        
        // Display properties customizer in the properties dialog
        _props.setPropertiesCustomizer( propsCustom );

        // Display layout customizer in the properties dialog
        _props.setLayoutCustomizer( layoutCustom );

        // Set the object for the properties customizer to manipulate
        if( propsCustom != null )
            propsCustom.setObject( c );

        // Set the object for the layout customizer to manipulate
        if( layoutCustom != null )
            layoutCustom.setObject( c );

        // Return whether or not we should open the dialog
        return ((c != null) && !(c instanceof DateablePanel));
    }
    
    
    private Customizer propsCustomizerFromComponent( Component c )
    {
        if( c == null )
            return null;

        Customizer custom = null;
        
        try
        {
            BeanInfo bi = Introspector.getBeanInfo( c.getClass() );
            BeanDescriptor bd = bi.getBeanDescriptor();

            // No BeanDescriptor defined - should never happen!
            if( bd == null )
                return null;

            Class cc = bd.getCustomizerClass();

            // No customizer defined
            if( cc == null )
                return null;

            custom = (Customizer) cc.newInstance();
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
        
        return custom;
    }
    
    
    private Customizer layoutCustomizerFromComponent( Component c )
    {
        if( (c == null) || (c instanceof CalendarPanel) )
            return null;
        
        Container parent = c.getParent();

        if( parent == null )
            return null;

        if( parent.getLayout() instanceof GridBagLayout )
            return _gblayoutCustom;
        else
            return _nulllayoutCustom;            
    }
    
    
    private String beanNameFromClass( Class beanClass )
    {
        String qualified = beanClass.getName();
        int lastdot = qualified.lastIndexOf( '.' );
        return ( lastdot != -1 ? qualified.substring(lastdot+1) : qualified );
    }
    
    
    private void actionNew( int kind )
    {
        CalendarPanel cp;
        
        switch( kind )
        {
        case BLANK_CALENDAR:
            cp = new CalendarPanel();
            break;
        case LIST_CALENDAR:
            cp = new CalendarListPanel();
            break;
        case PANORAMA_CALENDAR:
            cp = new CalendarPanoramaPanel();
            break;
        case TABLE_CALENDAR:
        default:
            cp = new CalendarTablePanel();
            break;
        case COVERSHEET:
            cp = new CalendarCoversheetPanel();
            break;
        }
        
        cp.addContainerListener( new ContainerAdapter() {
            public void componentAdded( ContainerEvent e ) {
                // Trick user into seeing no buttons selected
                noneBtn.setSelected( true );
            }
        } );
        
        panel.removeAll();
        panel.add( cp );
        panel.setNumPagesWidth( cp.getNumPagesWidth() );
        
        _calendar = cp;
        calcPageSize();               
        setDate( _startYear + yearCmb.getSelectedIndex(), monthCmb.getSelectedIndex() );
        
    }


    private void calcPageSize()
    {
        double topMargin = _pageFormat.getImageableY();
        double leftMargin = _pageFormat.getImageableX();
        double bottomMargin = _pageFormat.getHeight() - (_pageFormat.getImageableY() + _pageFormat.getImageableHeight());
        double rightMargin = _pageFormat.getWidth() - (_pageFormat.getImageableX() + _pageFormat.getImageableWidth());
        
        int pageWidth  = (int) (_pageFormat.getWidth() / panel.getNumPagesWidth() - (leftMargin + rightMargin));
        int pageHeight = (int) (_pageFormat.getHeight() - (topMargin + bottomMargin));
        
        System.out.println( "pageWidth = " + pageWidth + "; pageHeight = " + pageHeight );
        
        _pageSize = new Dimension( pageWidth, pageHeight );
        
        if( _calendar != null )
            _calendar.setActualSize( _pageSize );   // Resize the CalendarPanel
        
        panel.setActualSize( _pageSize );           // Resize containing DateablePanel
        panel.validate();
        panel.repaint();
    }
    
    
    private void updateActionsWithSelection( boolean haveSelection )
    {
        deleteAction.setEnabled( haveSelection );
        propertiesAction.setEnabled( haveSelection );
    }
    
    ////////////////////////////////////////////////////////////////////////////
    
    public class TransferableClassObject
        implements Transferable
    {
        private Object data;
        private DataFlavor classDataFlavor;
        
        public TransferableClassObject( Object data )
        {
            this.data = data;
            
            try {
                classDataFlavor = new DataFlavor( DataFlavor.javaJVMLocalObjectMimeType + "; class=java.lang.Class" );
            } catch( ClassNotFoundException e ) {
                e.printStackTrace();
            }
        }
        
        public Object getTransferData( DataFlavor dataFlavor )
            throws java.awt.datatransfer.UnsupportedFlavorException, java.io.IOException
        {
            if( !dataFlavor.equals( classDataFlavor ) )
                throw new UnsupportedFlavorException( dataFlavor );
                
            return data;
        }
        
        public DataFlavor[] getTransferDataFlavors()
        {
            return new DataFlavor[] { classDataFlavor };
        }
        
        public boolean isDataFlavorSupported( DataFlavor dataFlavor )
        {
            return dataFlavor.equals( classDataFlavor );
        }
    }
    
    private class MFDropSource
        implements DragGestureListener
    {
        private DragSource source;
        private DragGestureRecognizer recognizer;
        private AbstractButton button;
        private Image image;
        private Class data;
        
        public MFDropSource( AbstractButton b, Image i, Class d )
        {
            button = b;
            image = i;
            data = d;
            source = new DragSource();
            recognizer = source.createDefaultDragGestureRecognizer( button, DnDConstants.ACTION_COPY, this );
        }
        
        public void dragGestureRecognized( DragGestureEvent dge )
        {
            Transferable t = new TransferableClassObject( data );
            
            //System.out.println( "DragImageSupported: " + source.isDragImageSupported() );
            
            source.startDrag( dge, null, /*image, new Point(8,8),*/ t, null );
        }
    }
}
