/* *****************************************************************************
 *
 * Filename: ColorEditor.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  06 February 2003
 *
 * Purpose:  Editor bean for Colors.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.*;
import java.util.ResourceBundle;
import javax.swing.*;
import javax.accessibility.*;

/**
 *
 * @author  Robert
 */
public class ColorEditor
    extends com.tsac.calmaker.beans.EditorBase
    implements java.awt.event.FocusListener
{
    // Statics
    protected static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    // Properties
    protected Color _value = null;
    
    // Static State
    protected static JColorChooser _chooser = null;
    
    // State
    protected transient Image _image = null;
    protected transient ImageIcon _icon = null;
    protected transient String accessibleText = "";
    protected transient ComponentListener _updater;
    protected transient JDialog _colordlg = null;
    
    
    /** Creates new form BeanForm */
    public ColorEditor()
    {
        initComponents();

        // Only the component itself is focusable
        this.setFocusable( true );
        colorLbl.setFocusable( false );
        chooserBtn.setFocusable( false );
        
        // Add self as focus listener so we can draw focus
        this.addFocusListener( this );
        
        _updater = new ComponentAdapter() {
            public void componentShown( ComponentEvent e ) {
                if( _value != null )
                    updateColor( _value );                
            }
        };
        
        // Add action for opening the chooser to our action map
        ActionMap actions = this.getActionMap();
        actions.put( "openChooser", new OpenChooserAction() );
        
        // Bind action for opening the chooser to the Space key
        InputMap input = this.getInputMap();
        input.put( KeyStroke.getKeyStroke( KeyEvent.VK_SPACE, 0 ), "openChooser" );
    }
    
    
    /** From JComponent - set on label and pass on to super-class */
    public void setToolTipText( String text )
    {
        super.setToolTipText( text );
        this.getAccessibleContext().setAccessibleDescription( text );
        colorLbl.setToolTipText( text );
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        colorLbl = new javax.swing.JLabel();
        chooserBtn = new javax.swing.JButton();

        setLayout(new java.awt.GridBagLayout());

        colorLbl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/color.gif")));
        colorLbl.setText("[255,255,255]");
        colorLbl.setBorder(new javax.swing.border.EtchedBorder());
        colorLbl.setLabelFor(chooserBtn);
        colorLbl.setMinimumSize(new java.awt.Dimension(110, 22));
        colorLbl.setPreferredSize(new java.awt.Dimension(110, 22));
        colorLbl.addMouseListener(new java.awt.event.MouseAdapter()
        {
            public void mousePressed(java.awt.event.MouseEvent evt)
            {
                colorLblMousePressed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.ipadx = 4;
        add(colorLbl, gridBagConstraints);

        chooserBtn.setText("...");
        chooserBtn.setToolTipText(resources.getString("Color_Button_Tooltip"));
        chooserBtn.setMargin(new java.awt.Insets(0, 2, 0, 2));
        chooserBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                chooserBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        add(chooserBtn, gridBagConstraints);

    }//GEN-END:initComponents

    private void colorLblMousePressed(java.awt.event.MouseEvent evt)//GEN-FIRST:event_colorLblMousePressed
    {//GEN-HEADEREND:event_colorLblMousePressed
        this.requestFocus();
    }//GEN-LAST:event_colorLblMousePressed

    private void chooserBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_chooserBtnActionPerformed
    {//GEN-HEADEREND:event_chooserBtnActionPerformed
        actionColorButtonClicked();
    }//GEN-LAST:event_chooserBtnActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton chooserBtn;
    private javax.swing.JLabel colorLbl;
    // End of variables declaration//GEN-END:variables

    
    public void setValue( Color value )
    {
        Color oldval = _value;
        
        // De-bounce - otherwise we'll loop indefinitely
        if( value != oldval )
        {
            _value = value;            
            updateColor( value );
            firePropertyChange( "value", oldval, value );
        }
    }
    
    public Color getValue()  { return _value; }
    
    
    public void actionColorButtonClicked()
    {
        if( _chooser == null )
        {
            // Setup the chooser
            _chooser = new JColorChooser();
            _chooser.addChooserPanel( new GrayColorChooserPanel() );
            _chooser.addChooserPanel( new WebColorChooserPanel() );
        }
        
        if( _colordlg == null )
        {
            // Create the dialog
            _colordlg = JColorChooser.createDialog(
                this, resources.getString( "Color_Chooser" ), true, _chooser,
                new ActionListener() {
                    public void actionPerformed( ActionEvent e ) {
                        setValue( _chooser.getColor() );
                        fireActionEvent( new ActionEvent( this, ActionEvent.ACTION_PERFORMED, _value.toString() )  );
                    }
                }, null );
        }
        
        // Set starting color
        _chooser.setColor( _value );
        
        // Show the dialog (modal, so will block)
        _colordlg.setVisible( true );
    }
    
    
    protected void updateColor( Color c )
    {
        // Make sure the icon exists
        if( _image == null )
        {
            _image = createImage( 16, 16 );
            _icon = (_image != null) ? new ImageIcon( _image ) : null;
        }

        // Update the icon
        if( _image != null )
        {
            // Update the icon
            Graphics g = _image.getGraphics();
            g.setColor( c );
            g.fillRect( 0, 0, 16, 16 );
            g.setColor( Color.black );
            g.drawRect( 0, 0, 15, 15 );
        }
            
        colorLbl.setIcon( _icon );
        
        // Update the text
        colorLbl.setText( "[" + c.getRed() + "," + c.getGreen() + "," + c.getBlue() + "]" );
        
        // Update the accessible name?
        accessibleText = c.getRed() + " " + c.getGreen() + " " + c.getBlue();
        
        // Force a repaint
        colorLbl.repaint();
    }
    
    
    // Overridden
    public void addNotify()
    {
        super.addNotify();
        getTopLevelAncestor().addComponentListener( _updater );
    }

    // Overridden
    public void removeNotify()
    {
        super.removeNotify();
        getTopLevelAncestor().removeComponentListener( _updater );
    }
        
    // Overridden
    public void paint( Graphics g )
    {
        super.paint( g );
        
        // Paint the focus (labels don't normally show focus
        if( this.hasFocus() )
        {
            g.setColor( UIManager.getColor( "Button.focus" ) );
            g.drawRect( colorLbl.getX() + 3, colorLbl.getY() + 3, colorLbl.getWidth() - 7, colorLbl.getHeight() - 7 );
        }
    }
    
    public void focusGained( FocusEvent e )  { repaint(); }
    
    public void focusLost( FocusEvent e )  { repaint(); }

    
    ////////////////////////////////////////////////////////////////////////////
    // Inner Classes
    
    /**
     * Simple Action class for opening the color chooser.  Just defers to the
     * existing method which does the work.
     */
    private class OpenChooserAction
        extends AbstractAction
    {
        public void actionPerformed( ActionEvent e )
        {
            actionColorButtonClicked();
        }
    }
    
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility
    
    public class AccessibleColorEditor
        extends AccessibleJPanel
        implements AccessibleAction, AccessibleText
    {
        // Return the AccessibleAction (us)
        public AccessibleAction getAccessibleAction()  { return this; }

        // Hide all the children
        public int getAccessibleChildrenCount()  { return 0; }
        
        // Should never be called
        public Accessible getAccessibleChild( int n )  { return null; }
        
        // Return our role
        public AccessibleRole getAccessibleRole()
        {
            return AccessibleRole.TEXT;
        }

        // Return the AccessibleText (us)
        public AccessibleText getAccessibleText()  { return this; }
        
        ///////////////////
        // AccessibleAction
        
        // From AccessibleAction - return number of actions
        public int getAccessibleActionCount()  { return 1; }
        
        // From AccessibleAction - return our one and only action if n is 0
        public String getAccessibleActionDescription( int n )
        {
            return ( n == 0 ? "openChooser" : null );
        }
        
        // From AccessibleAction - perform our one and only action
        public boolean doAccessibleAction( int n )
        {
            if( n != 0 )  return false;

            actionColorButtonClicked();
            return true;
        }
        
        /////////////////
        // AccessibleText
        
        // From AccessibleText - return character or whole sentence if at start
        public String getAfterIndex( int part, int index )
        {
            switch( part )
            {
            case AccessibleText.CHARACTER:
                if( (index >= 0) && (index + 1 < accessibleText.length()) )
                    return accessibleText.substring( index + 1, index + 2 );
                else
                    return null;
            case AccessibleText.WORD:
            case AccessibleText.SENTENCE:
                return (index <= 0) ? accessibleText : null;
            }
            
            return null;
        }
        
        // From AccessibleText - return character or whole sentence
        public String getAtIndex( int part, int index )
        {
            switch( part )
            {
            case AccessibleText.CHARACTER:
                if( (index >= 0) && (index < accessibleText.length()) )
                    return accessibleText.substring( index, index + 1 );
                else
                    return null;
            case AccessibleText.WORD:
            case AccessibleText.SENTENCE:
                return accessibleText;
            }
            
            return null;
        }
        
        // From AccessibleText - return character or whole sentence if at end
        public String getBeforeIndex( int part, int index )
        {
            switch( part )
            {
            case AccessibleText.CHARACTER:
                if( (index >= 1) && (index - 1 < accessibleText.length()) )
                    return accessibleText.substring( index - 1, index );
                else
                    return null;
            case AccessibleText.WORD:
            case AccessibleText.SENTENCE:
                return (index >= accessibleText.length()) ? accessibleText : null;
            }
            
            return null;
        }
        
        // From AccessibleText - don't support caret, pretend at end
        public int getCaretPosition()  { return (accessibleText.length() - 1); }
        
        // From AccessibleText - return length of string
        public int getCharCount()  { return accessibleText.length(); }
        
        // From AccessibleText - don't support attributes
        public javax.swing.text.AttributeSet getCharacterAttribute( int index )
        {
            return null;
        }
        
        // From AccessibleText - calculate rough box around character
        public java.awt.Rectangle getCharacterBounds( int index )
        {            
            if( index > accessibleText.length() - 1 )
                return null;
            
            java.awt.FontMetrics metrics = getFontMetrics( getFont() );
            int hpos = metrics.stringWidth( accessibleText.substring( 0, index ) );
            int width = metrics.stringWidth( accessibleText.substring( index, index + 1 ) );
            return new java.awt.Rectangle( hpos + 16, 0, width, colorLbl.getHeight() );
        }
        
        // From AccessibleText - calculate horizontal position in string
        public int getIndexAtPoint(java.awt.Point point)
        {
            java.awt.FontMetrics metrics = getFontMetrics( getFont() );
            
            for( int index = 0; index < accessibleText.length(); index++ )
            {
                if( metrics.stringWidth( accessibleText.substring( 0, index + 1 ) ) + 16 > point.x )
                    return index;
            }
            
            return -1;
        }
        
        // From AccessibleText - We don't support selections
        public String getSelectedText()  { return null; }
        
        // From AccessibleText - We don't support selections
        public int getSelectionEnd()  { return -1; }
        
        // From AccessibleText - We don't support selections
        public int getSelectionStart()  { return -1; }
    }
    
    
    public AccessibleContext getAccessibleContext()
    {
        if( accessibleContext == null )
            accessibleContext = new AccessibleColorEditor();
        
        return accessibleContext;
    }    
}
