/* *****************************************************************************
 *
 * Filename: StaticTitle.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  08 February 2002
 *
 * Purpose:  Bean for displaying a fixed text string.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;


public class StaticTitle
    extends DateableTitle
{
    // Contructor
	public StaticTitle()
	{
        _text = "Text";
	}

    
    // From Title
    public boolean isTextSupported()  { return true; }
    
    
    // Overrides default behavior to size largest month
    public Dimension getPreferredSize()
    {
        return new Dimension(
            getFontMetrics( _font ).stringWidth( _text ),
            getFontMetrics( _font ).getHeight() );
    }


    // Overrides default paint method
    public void paint( Graphics og, double scale )
    {
        if( _text == null )
            return;
        
        Graphics2D g = (Graphics2D) og;
        Font font = _font.deriveFont( (float) (scale * _font.getSize()) );
        FontMetrics metrics = getFontMetrics( font );
        float hpos;
        
        switch( _justification )
        {
        default:
        case JUSTIFY_LEFT:
            hpos = 0;
            break;
        case JUSTIFY_CENTER:
            hpos = (getWidth() - metrics.stringWidth(_text)) / (float) 2;
            break;
        case JUSTIFY_RIGHT:
            hpos = getWidth() - metrics.stringWidth(_text);
            break;
        }
        
        g.setColor( _color );
        g.setFont( font );
        g.drawString( _text, hpos, (float) metrics.getAscent() );
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff
    
    protected String getAccessibleName()
    {
        return resources.getString( "StaticTitle_Name" );
    }
    
    protected String getAccessibleDescription()
    {
        return resources.getString( "StaticTitle_Desc" );
    }
}