/* *****************************************************************************
 *
 * Filename: TitleCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  18 December 2002, 17:48
 *
 * Purpose:  Customizer for all title beans.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import java.beans.*;
import javax.swing.ImageIcon;
import javax.swing.JColorChooser;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class TitleCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer
{
    // Statics
    private static final ResourceBundle resources = ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans");
    
    // State
    private Title _object;
    
    // Cached
    private PropertyChangeSupport _pcsupport = null;
    
    
    /** Creates new customizer TitleCustomizer */
    public TitleCustomizer()
    {
        initComponents();
        
        // Accessibility stuff
        textTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Text_Tooltip" ) );
        
        Utilities.setAccessibleLabelNames( this );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        textPropsPnl = new javax.swing.JPanel();
        fontLbl = new javax.swing.JLabel();
        fontEdit = new com.tsac.calmaker.beans.FontEditor();
        colorLbl = new javax.swing.JLabel();
        colorEdit = new com.tsac.calmaker.beans.ColorEditor();
        justifyLbl = new javax.swing.JLabel();
        justifiyEdit = new com.tsac.calmaker.beans.JustificationEditor();
        textLbl = new javax.swing.JLabel();
        textTxt = new javax.swing.JTextField();

        setLayout(new java.awt.GridBagLayout());

        textPropsPnl.setLayout(new java.awt.GridBagLayout());

        textPropsPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), resources.getString("Title")));
        fontLbl.setText(resources.getString("Font_"));
        fontLbl.setDisplayedMnemonic(resources.getString( "Font_Mnemonic" ).charAt(0));
        fontLbl.setLabelFor(fontEdit);
        fontLbl.setMinimumSize(new java.awt.Dimension(36, 16));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 2, 2);
        textPropsPnl.add(fontLbl, gridBagConstraints);

        fontEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                fontEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 2, 4);
        textPropsPnl.add(fontEdit, gridBagConstraints);

        colorLbl.setText(resources.getString("Color_"));
        colorLbl.setDisplayedMnemonic(resources.getString( "Color_Mnemonic" ).charAt(0));
        colorLbl.setLabelFor(colorEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        textPropsPnl.add(colorLbl, gridBagConstraints);

        colorEdit.setToolTipText(resources.getString("Color_Tooltip"));
        colorEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                colorEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        textPropsPnl.add(colorEdit, gridBagConstraints);

        justifyLbl.setText(resources.getString("Justify_"));
        justifyLbl.setDisplayedMnemonic(resources.getString( "Justify_Mnemonic" ).charAt(0));
        justifyLbl.setLabelFor(justifiyEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        textPropsPnl.add(justifyLbl, gridBagConstraints);

        justifiyEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                justifiyEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        textPropsPnl.add(justifiyEdit, gridBagConstraints);

        textLbl.setText(resources.getString("Text_"));
        textLbl.setDisplayedMnemonic(resources.getString( "Text_Mnemonic" ).charAt(0));
        textLbl.setLabelFor(textTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 4, 2);
        textPropsPnl.add(textLbl, gridBagConstraints);

        textTxt.setColumns(24);
        textTxt.setToolTipText(resources.getString("Text_Tooltip"));
        textTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                textTxtActionPerformed(evt);
            }
        });

        textTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                textTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 4);
        textPropsPnl.add(textTxt, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        add(textPropsPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void textTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_textTxtFocusLost
    {//GEN-HEADEREND:event_textTxtFocusLost
        actionTextChanged();
    }//GEN-LAST:event_textTxtFocusLost

    private void textTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_textTxtActionPerformed
    {//GEN-HEADEREND:event_textTxtActionPerformed
        actionTextChanged();
    }//GEN-LAST:event_textTxtActionPerformed

    private void colorEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_colorEditActionPerformed
    {//GEN-HEADEREND:event_colorEditActionPerformed
        actionColorChanged();
    }//GEN-LAST:event_colorEditActionPerformed

    private void fontEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_fontEditActionPerformed
    {//GEN-HEADEREND:event_fontEditActionPerformed
        actionFontChanged();
    }//GEN-LAST:event_fontEditActionPerformed

    private void justifiyEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_justifiyEditActionPerformed
    {//GEN-HEADEREND:event_justifiyEditActionPerformed
        actionJustificationChanged();
    }//GEN-LAST:event_justifiyEditActionPerformed

    public void setObject(Object obj)
    {        
        if( obj instanceof Title )
        {
            Title t = (Title) obj;
            _object = t;
            _pcsupport = new PropertyChangeSupport( t );

            updateColor( t.getColor() );
            updateFont( t.getFont() );
            updateJustification( t.getJustification() );
            updateText( t.getText() );
        }
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private com.tsac.calmaker.beans.FontEditor fontEdit;
    private javax.swing.JPanel textPropsPnl;
    private javax.swing.JLabel fontLbl;
    private com.tsac.calmaker.beans.JustificationEditor justifiyEdit;
    private javax.swing.JLabel colorLbl;
    private com.tsac.calmaker.beans.ColorEditor colorEdit;
    private javax.swing.JLabel textLbl;
    private javax.swing.JLabel justifyLbl;
    private javax.swing.JTextField textTxt;
    // End of variables declaration//GEN-END:variables

    
    public void addPropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.addPropertyChangeListener( l );
    }

    
    public void removePropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.removePropertyChangeListener( l );
    }

    
    protected void updateColor( Color c )
    {
        colorEdit.setValue( c );
    }
    
    
    protected void updateFont( Font f )
    {
        fontEdit.setValue( f );
    }
    
    
    protected void actionFontChanged()
    {
        Font oldFont = _object.getFont();
        Font newFont = fontEdit.getValue();

        _object.setFont( newFont );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "font", oldFont, newFont );
    }
    
    
    protected void actionColorChanged()
    {
        Color oldColor = _object.getColor();
        Color newColor = colorEdit.getValue();

        _object.setColor( newColor );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "color", oldColor, newColor );            
    }
    
    
    protected void updateJustification( int justification )
    {
        justifiyEdit.setValue( justification );
    }
    
    
    protected void actionJustificationChanged()
    {
        int oldValue = _object.getJustification();
        int newValue = justifiyEdit.getValue();
        
        _object.setJustification( newValue );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "justification", oldValue, newValue );
    }
    
    
    protected void updateText( String t )
    {
        // Updated the Text textbox
        textTxt.setText( t != null ? t : "" );
        
        // Disable if Text is not supported
        boolean supported = _object.isTextSupported();
        textLbl.setEnabled( supported );
        textTxt.setEditable( supported );
        textTxt.setEnabled( supported );
    }
    
    protected void actionTextChanged()
    {
        String oldText = _object.getText();
        String newText = textTxt.getText();
        
        // Text is always stored as null if it is an empty string
        if( (newText != null) && (newText.length() == 0) )
            newText = null;
        
        _object.setText( newText );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "text", oldText, newText );
    }
}
