/* *****************************************************************************
 *
 * Filename: WebColorChooserPanel.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  10 February 2003
 *
 * Purpose:  A ColorChooser panel for pick colours by Web #rrggbb.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import java.text.DecimalFormat;
import java.util.ResourceBundle;
import javax.swing.JButton;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.KeyStroke;

/**
 *
 * @author  Robert
 */
public class WebColorChooserPanel
    extends javax.swing.colorchooser.AbstractColorChooserPanel
{
    // Statics
    protected static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    protected static Object[] colors = {
        Color.red, "Red", Color.pink, "Pink",
        Color.orange, "Orange", Color.yellow, "Yellow",
        Color.green, "Green", Color.cyan, "Cyan",
        Color.blue, "Blue", Color.magenta, "Magenta",
        Color.white, "White", Color.lightGray, "Light Gray", Color.gray, "Gray",
        Color.darkGray, "Dark Gray", Color.black, "Black"
    };
    
    
    /** Creates new form BeanForm */
    public WebColorChooserPanel()
    {
        initComponents();
        
        for( int n = 0; n < colors.length; n+=2 )
            palettePnl.add( createPaletteButton( (Color) colors[n], (String) colors[n+1] ) );
        
        // Create actions for moving focus round the palette
        ActionMap actions = this.getActionMap();
        actions.put( "navigateLeft",  new GridNavigateAction( palettePnl, -1, 0 ) );
        actions.put( "navigateRight", new GridNavigateAction( palettePnl, +1, 0 ) );
        actions.put( "navigateUp",    new GridNavigateAction( palettePnl, 0, -1 ) );
        actions.put( "navigateDown",  new GridNavigateAction( palettePnl, 0, +1 ) );
        
        // Bind actions to cursor-keys 
        InputMap input = this.getInputMap( WHEN_ANCESTOR_OF_FOCUSED_COMPONENT );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0), "navigateLeft" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0), "navigateRight" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_UP,0), "navigateUp" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,0), "navigateDown" );

        // Accessibility stuff
        webTxt.getAccessibleContext().setAccessibleName( resources.getString( "Web_Color_Name" ) );
        webTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Web_Color_Tooltip" ) );
        redTxt.getAccessibleContext().setAccessibleName( resources.getString( "Red_Name" ) );
        redTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Red_Tooltip" ) );
        greenTxt.getAccessibleContext().setAccessibleName( resources.getString( "Green_Name" ) );
        greenTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Green_Tooltip" ) );
        blueTxt.getAccessibleContext().setAccessibleName( resources.getString( "Blue_Name" ) );
        blueTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Blue_Tooltip" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        palettePnl = new javax.swing.JPanel();
        inputPnl = new javax.swing.JPanel();
        webTxt = new javax.swing.JTextField();
        hashLbl = new javax.swing.JLabel();
        rgbPnl = new javax.swing.JPanel();
        redLbl = new javax.swing.JLabel();
        redTxt = new javax.swing.JTextField();
        greenLbl = new javax.swing.JLabel();
        greenTxt = new javax.swing.JTextField();
        blueLbl = new javax.swing.JLabel();
        blueTxt = new javax.swing.JTextField();

        setLayout(new java.awt.GridBagLayout());

        palettePnl.setLayout(new java.awt.GridLayout(2, 8));

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        add(palettePnl, gridBagConstraints);

        inputPnl.setLayout(new java.awt.GridBagLayout());

        webTxt.setColumns(6);
        webTxt.setToolTipText(resources.getString("Web_Color_Tooltip"));
        webTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                webTxtActionPerformed(evt);
            }
        });

        webTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                webTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 0);
        inputPnl.add(webTxt, gridBagConstraints);

        hashLbl.setText("#");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 2);
        inputPnl.add(hashLbl, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(48, 0, 0, 0);
        add(inputPnl, gridBagConstraints);

        rgbPnl.setLayout(new java.awt.GridBagLayout());

        redLbl.setText(resources.getString("Red"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 2);
        rgbPnl.add(redLbl, gridBagConstraints);

        redTxt.setColumns(3);
        redTxt.setEditable(false);
        redTxt.setToolTipText(resources.getString("Red_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 8);
        rgbPnl.add(redTxt, gridBagConstraints);

        greenLbl.setText(resources.getString("Green"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 2);
        rgbPnl.add(greenLbl, gridBagConstraints);

        greenTxt.setColumns(3);
        greenTxt.setEditable(false);
        greenTxt.setToolTipText(resources.getString("Green_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 8);
        rgbPnl.add(greenTxt, gridBagConstraints);

        blueLbl.setText(resources.getString("Blue"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 2);
        rgbPnl.add(blueLbl, gridBagConstraints);

        blueTxt.setColumns(3);
        blueTxt.setEditable(false);
        blueTxt.setToolTipText(resources.getString("Blue_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 0);
        rgbPnl.add(blueTxt, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(48, 0, 0, 0);
        add(rgbPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void webTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_webTxtFocusLost
    {//GEN-HEADEREND:event_webTxtFocusLost
        actionWebChanged();
    }//GEN-LAST:event_webTxtFocusLost

    private void webTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_webTxtActionPerformed
    {//GEN-HEADEREND:event_webTxtActionPerformed
        actionWebChanged();
    }//GEN-LAST:event_webTxtActionPerformed

    
    protected void buildChooser()
    {
    }    
    
    
    public String getDisplayName()
    {
        return resources.getString( "Web" );
    }    
    
    
    public javax.swing.Icon getLargeDisplayIcon()
    {
        return null;
    }
    
    public javax.swing.Icon getSmallDisplayIcon()
    {
        return null;
    }

    
    public void updateChooser()
    {
        int[] rgb = decodeColor( getColorSelectionModel().getSelectedColor() );
        
        updateWeb( rgb );
        updateRGB( rgb );
    }

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel blueLbl;
    private javax.swing.JTextField webTxt;
    private javax.swing.JTextField blueTxt;
    private javax.swing.JLabel greenLbl;
    private javax.swing.JPanel palettePnl;
    private javax.swing.JTextField greenTxt;
    private javax.swing.JLabel redLbl;
    private javax.swing.JPanel rgbPnl;
    private javax.swing.JTextField redTxt;
    private javax.swing.JLabel hashLbl;
    private javax.swing.JPanel inputPnl;
    // End of variables declaration//GEN-END:variables

    
    protected JButton createPaletteButton( Color c, String name )
    {
        JButton btn = new JButton();
        btn.setMinimumSize( new Dimension( 16, 16 ) );
        btn.setPreferredSize( new Dimension( 16, 16 ) );
        btn.setBackground( c );
        btn.setToolTipText( name + " " + c.getRed() + " " + c.getGreen() + " " + c.getBlue() );
        btn.getAccessibleContext().setAccessibleName( name );
        btn.addActionListener( new ColorAction( c ) );
        return btn;
    }
    
    
    protected static StringBuffer toHexString( int n, StringBuffer buf )
    {
        String hex = Integer.toHexString( n );
        
        if( hex.length() < 2 )
            buf.append( '0' );
        
        buf.append( hex );
        
        return buf;
    }
    
    
    protected static int[] decodeString( String web )
        throws NumberFormatException
    {
        if( web.length() != 6 )
            throw new NumberFormatException( "Need 6 hex digits" );
        
        String red   = web.substring( 0, 2 );
        String green = web.substring( 2, 4 );
        String blue  = web.substring( 4, 6 );
        
        int[] rgb = new int[3];
        rgb[0] = Integer.parseInt( red, 16 );
        rgb[1] = Integer.parseInt( green, 16 );
        rgb[2] = Integer.parseInt( blue, 16 );
        
        return rgb;
    }

    
    protected static int[] decodeColor( Color c )
    {
        int[] rgb = new int[3];

        rgb[0] = c.getRed();
        rgb[1] = c.getGreen();
        rgb[2] = c.getBlue();
        
        return rgb;
    }
    

    protected void updateWeb( int[] rgb )
    {
        String hex =
            toHexString( rgb[2],
                toHexString( rgb[1],
                    toHexString( rgb[0], new StringBuffer() ) ) ).toString().toUpperCase();
        
        if( !hex.equals( webTxt.getText() ) )
            webTxt.setText( hex );
    }
    
    
    protected void actionWebChanged()
    {
        int[] rgb;

        try
        {
            rgb = decodeString( webTxt.getText() );
            getColorSelectionModel().setSelectedColor( new Color( rgb[0], rgb[1], rgb[2] ) );
        }
        catch( NumberFormatException e )
        {
            rgb = decodeColor( getColorSelectionModel().getSelectedColor() );
            updateWeb( rgb );
        }
        
        updateRGB( rgb );
    }
        
    
    protected void updateRGB( int[] rgb )
    {
        redTxt.setText( Integer.toString( rgb[0] ) );
        greenTxt.setText( Integer.toString( rgb[1] ) );
        blueTxt.setText( Integer.toString( rgb[2] ) );
    }
    
    
    protected class ColorAction
        implements ActionListener
    {
        private Color _color;
        
        public ColorAction( Color c )
        {
            _color = c;
        }
        
        public void actionPerformed(java.awt.event.ActionEvent actionEvent)
        {
            getColorSelectionModel().setSelectedColor( _color );
        }
    }
}
